﻿package
{
	import Box2D.Collision.*;
	import Box2D.Collision.Shapes.*;
	import Box2D.Common.*;
	import Box2D.Common.Math.*;
	import Box2D.Dynamics.*;
	import Box2D.Dynamics.Contacts.*;
	import Box2D.Dynamics.Joints.*;
	
	import flash.display.*;
	import flash.events.*;
	
	public class CGameMain extends MovieClip
	{
		private var _b2World:b2World;
		private var _b2Gravity:b2Vec2;
		
		public function CGameMain()
		{				
			createB2World();			
			createStageWall();			
			initBox2DDebugDraw();
			
			this.addEventListener( Event.ENTER_FRAME, onEnterFrame );
			
			createDynamicBoxAndCircle();
			//createDistanceJoint();
			//CreateRevoluteJoint();
			//CreatePrismaticJoint();
			//CreatePulleyJoint();
		}
		
		public function initBox2DDebugDraw():void
		{
			var debug_draw:b2DebugDraw = new b2DebugDraw();			
			var debug_sprite:Sprite = new Sprite();			
			debug_draw.SetSprite(debug_sprite);			
			debug_draw.SetDrawScale(32);			
			debug_draw.SetFlags(b2DebugDraw.e_shapeBit);			
			_b2World.SetDebugDraw(debug_draw);			
			addChild(debug_sprite);		
		}
		
		public function createStaticBox(x:Number, y:Number, sizex:Number, sizey:Number):void
		{
			var bodyDef:b2BodyDef = new b2BodyDef();
			bodyDef.position.Set(x/32, y/32);
			
			var box:b2PolygonShape = new b2PolygonShape();
			box.SetAsBox(sizex/32, sizey/32);
			
			var fixtureDef:b2FixtureDef = new b2FixtureDef();
			fixtureDef.shape = box;
			
			var body:b2Body = _b2World.CreateBody(bodyDef);
			body.CreateFixture(fixtureDef);				
		}
		
		public function createStageWall():void
		{
			createStaticBox(16, 240, 16, 280);
			createStaticBox(304, 240, 16, 280);
			createStaticBox(160, 466, 160, 16);
		}
		
		public function createB2World():void
		{
			_b2Gravity = new b2Vec2(0, 98/32);
			_b2World = new b2World(_b2Gravity, true);
			
			_b2World.SetContactListener(new MyContactListener());
		}		
		
		//////////////////////////////////////////////////////////////////////////////////////////////////
		//// 예제 1
		
		public function createDynamicBox(x:Number, y:Number, sizex:Number, sizey:Number):void
		{
			var bodyDef:b2BodyDef = new b2BodyDef();
			bodyDef.position.Set(x/32, y/32);
			bodyDef.type = b2Body.b2_dynamicBody;
			bodyDef.angularDamping = 0.2;
			
			var box:b2PolygonShape = new b2PolygonShape();
			box.SetAsBox(sizex/32, sizey/32);
			
			var fixtureDef:b2FixtureDef = new b2FixtureDef();
			fixtureDef.shape = box;
			fixtureDef.density = 1;
			fixtureDef.friction = 0.1;
			
			var massData:b2MassData = new b2MassData();
			box.ComputeMass(massData, 1);
			
			var body:b2Body = _b2World.CreateBody(bodyDef);
			body.CreateFixture(fixtureDef);		
			body.SetMassData(massData);
		}
		
		public function createDynamicCircle(x:Number, y:Number, radius:Number)
		{
			var bodyDef:b2BodyDef = new b2BodyDef();
			bodyDef.position.Set(x/32, y/32);
			bodyDef.type = b2Body.b2_dynamicBody;
			bodyDef.angularDamping = 0.2;
			
			var circle:b2CircleShape = new b2CircleShape();				
			circle.SetRadius(radius/32);
			
			var fixtureDef:b2FixtureDef = new b2FixtureDef();
			fixtureDef.shape = circle;
			fixtureDef.density = 1;
			fixtureDef.friction = 0.1;
			
			var massData:b2MassData = new b2MassData();
			circle.ComputeMass(massData, 1);
			
			var body:b2Body = _b2World.CreateBody(bodyDef);
			body.CreateFixture(fixtureDef);
			body.SetMassData(massData);
		}
		
		public function createDynamicBoxAndCircle()
		{
			createDynamicBox(100, 0, 10, 10);
			createDynamicCircle(200, 0, 10);
		}
		
		//////////////////////////////////////////////////////////////////////////////////////////////////
		// 예제 2
		
		public function createDistanceJoint():void
		{
			var body:Array = new Array();
			
			var bodyDef:Array = new Array();
			bodyDef.push(new b2BodyDef());
			bodyDef.push(new b2BodyDef());
			
		 	var box:Array = new Array();
			box.push(new b2PolygonShape());
			box.push(new b2PolygonShape());
			
			box[0].SetAsBox(10/32, 10/32);
			box[1].SetAsBox(10/32, 10/32);
			 	
			var fixtureDef:Array = new Array();
			fixtureDef.push(new b2FixtureDef());
			fixtureDef.push(new b2FixtureDef());
			 
			fixtureDef[0].shape = box[0];
			fixtureDef[1].shape = box[1];
			 	
			bodyDef[0].type = b2Body.b2_staticBody;
			bodyDef[0].position.Set(160/32, 80/32);	
			body.push(_b2World.CreateBody(bodyDef[0]));
			body[0].CreateFixture(fixtureDef[0]);
			 	
			bodyDef[1].type = b2Body.b2_dynamicBody;
			bodyDef[1].position.Set(80/32, 0/32);	
			body.push(_b2World.CreateBody(bodyDef[1]));
			body[1].CreateFixture(fixtureDef[1]);
			
			var jointDef:b2DistanceJointDef = new b2DistanceJointDef();
			jointDef.Initialize(body[0], body[1], new b2Vec2(160/32, 80/32), new b2Vec2(80/32, 0/32));
			jointDef.collideConnected = true;
			jointDef.frequencyHz = 4.0;
			jointDef.dampingRatio = 0.1;
			 	
			var joint:b2Joint = _b2World.CreateJoint(jointDef);
		}
		
		//////////////////////////////////////////////////////////////////////////////////////////////////
		// 예제 3
		
		public function CreateRevoluteJoint():void
		{
			var body:Array = new Array();
			
			var bodyDef:Array = new Array();
			bodyDef.push(new b2BodyDef());
			bodyDef.push(new b2BodyDef());
			
			var box:Array = new Array();
			box.push(new b2PolygonShape());
			box.push(new b2PolygonShape());
			
			box[0].SetAsBox(10/32, 50/32);
			box[1].SetAsBox(50/32, 10/32);
			
			var fixtureDef:Array = new Array();
			fixtureDef.push(new b2FixtureDef());
			fixtureDef.push(new b2FixtureDef());
			
			fixtureDef[0].shape = box[0];
			fixtureDef[0].density = 1;
			fixtureDef[1].shape = box[1];
			fixtureDef[1].density = 1;
			
			bodyDef[0].type = b2Body.b2_staticBody;
			bodyDef[0].position.Set(160/32, 80/32);	
			body.push(_b2World.CreateBody(bodyDef[0]));
			body[0].CreateFixture(fixtureDef[0]);
			
			bodyDef[1].type = b2Body.b2_dynamicBody;
			bodyDef[1].position.Set(200/32, 130/32);	
			body.push(_b2World.CreateBody(bodyDef[1]));
			body[1].CreateFixture(fixtureDef[1]);
			
			var jointDef:b2RevoluteJointDef = new b2RevoluteJointDef();
			jointDef.Initialize(body[0], body[1], new b2Vec2(160/32, 130/32));
			jointDef.collideConnected = false;
			
			var joint:b2Joint = _b2World.CreateJoint(jointDef);
		}
		
		//////////////////////////////////////////////////////////////////////////////////////////////////
		// 예제 4
		
		public function CreatePrismaticJoint():void
		{
			var body:Array = new Array();
			
			var bodyDef:Array = new Array();
			bodyDef.push(new b2BodyDef());
			bodyDef.push(new b2BodyDef());
			
			var box:Array = new Array();
			box.push(new b2PolygonShape());
			box.push(new b2PolygonShape());
			
			box[0].SetAsBox(10/32, 50/32);
			box[1].SetAsBox(50/32, 10/32);
			
			var fixtureDef:Array = new Array();
			fixtureDef.push(new b2FixtureDef());
			fixtureDef.push(new b2FixtureDef());
			
			fixtureDef[0].shape = box[0];
			fixtureDef[0].density = 1;
			fixtureDef[1].shape = box[1];
			fixtureDef[1].density = 1;
			
			bodyDef[0].type = b2Body.b2_staticBody;
			bodyDef[0].position.Set(160/32, 80/32);	
			body.push(_b2World.CreateBody(bodyDef[0]));
			body[0].CreateFixture(fixtureDef[0]);
			
			bodyDef[1].type = b2Body.b2_dynamicBody;
			bodyDef[1].position.Set(160/32, 80/32);
			bodyDef[1].angle = 0.25 * b2Settings.b2_pi;
			body.push(_b2World.CreateBody(bodyDef[1]));
			body[1].CreateFixture(fixtureDef[1]);
						
			var jointDef:b2PrismaticJointDef = new b2PrismaticJointDef();
			jointDef.Initialize(body[0], body[1], new b2Vec2(160/32, 80/32), new b2Vec2(0, 1));
			jointDef.lowerTranslation = -200/32;
			jointDef.upperTranslation = 200/32;
			jointDef.enableLimit = true;
			
			jointDef.collideConnected = false;
			
			var joint:b2Joint = _b2World.CreateJoint(jointDef);
		}
		
		public function CreatePulleyJoint():void
		{
			var body:Array = new Array();
			
			var bodyDef:Array = new Array();
			bodyDef.push(new b2BodyDef());
			bodyDef.push(new b2BodyDef());
			
			var box:Array = new Array();
			box.push(new b2PolygonShape());
			box.push(new b2PolygonShape());
			
			box[0].SetAsBox(20/32, 20/32);
			box[1].SetAsBox(30/32, 30/32);
			
			var fixtureDef:Array = new Array();
			fixtureDef.push(new b2FixtureDef());
			fixtureDef.push(new b2FixtureDef());
			
			fixtureDef[0].shape = box[0];
			fixtureDef[0].density = 1;
			fixtureDef[1].shape = box[1];
			fixtureDef[1].density = 1;
			
			bodyDef[0].type = b2Body.b2_dynamicBody;
			bodyDef[0].position.Set(80/32, 160/32);	
			body.push(_b2World.CreateBody(bodyDef[0]));
			body[0].CreateFixture(fixtureDef[0]);
			
			bodyDef[1].type = b2Body.b2_dynamicBody;
			bodyDef[1].position.Set(240/32, 160/32);			
			body.push(_b2World.CreateBody(bodyDef[1]));
			body[1].CreateFixture(fixtureDef[1]);
			
			var ratio:Number = 1;
			var jointDef:b2PulleyJointDef = new b2PulleyJointDef();
			jointDef.Initialize(body[0], body[1], new b2Vec2(80/32, 10/32), new b2Vec2(240/32, 10/32),
								new b2Vec2(80/32, 160/32), new b2Vec2(240/32, 160/32), ratio);
			
			jointDef.maxLengthA = 200;
			jointDef.maxLengthB = 200;
			
			var joint:b2Joint = _b2World.CreateJoint(jointDef);
		}
		
		private function onEnterFrame(event:Event):void	
		{		
			_b2World.Step(1/30, 10, 10);			
			_b2World.ClearForces();			
			_b2World.DrawDebugData();
			
			/*
			var count:Number = _b2World.GetContactCount();
			var c:b2Contact = _b2World.GetContactList();
			for(var i:Number = 0; i<count; i++)
			{
				var manifold:b2Manifold = c.GetManifold();
								
				var worldmanifold:b2WorldManifold = new b2WorldManifold;
				c.GetWorldManifold(worldmanifold);
				
				for(var j:Number=0;j<manifold.m_pointCount;j++)
					trace("X = " + worldmanifold.m_points[j].x + ", Y = " + worldmanifold.m_points[j].y);
				
				c = c.GetNext();				
			}
			*/
		}
	}
}