﻿package
{
	import Box2D.Collision.Shapes.*;
	import Box2D.Common.Math.*;
	import Box2D.Dynamics.*;
	
	import flash.display.MovieClip;
	import flash.events.*;
	import flash.utils.*;
	import flash.net.*;
	
	public class CGameMain extends MovieClip
	{
		private var _b2World:b2World;
		private var _b2Gravity:b2Vec2;
		
		private var _b2CoinContainer:Array = new Array();
		private var _coinContainer:Array = new Array();
		private var _effectContainer:Array = new Array();
		
		
		private var _levelTimeDelay:Number = 1000;
		
		private var GAMEMODE_TITLE:Number = 1;
		private var GAMEMODE_GAME:Number = 2;
		private var GAMEMODE_GAMEOVER:Number = 3;
		
		private var _mode:Number = GAMEMODE_TITLE;
		
		private var _score:Number = 0;
		
		// 페이스북 아이디 가져오기
		var paramObj:Object = stage.loaderInfo.parameters;
		var _facebookuid:String = paramObj.facebookuid;
		
		public function CGameMain()
		{			
			super();
						
			createB2World();
			createStageWall();	
			
			this.addEventListener( Event.ENTER_FRAME, onEnterFrame );
			
			stage.addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
			stage.addEventListener(MouseEvent.MOUSE_UP, onMouseUp);			
		}
		
		private function setGame()
		{
			setTimer();
			
			gotoAndStop(1, "Game");
			
			for(var i:Number=0;i<16;i++)
				createCoin(Math.random()*256+32, -(i/8) * 16, Math.round(Math.random()*5+1));
			
			_levelTimeDelay = 1000;
			_score = 0;			
			_scoreNumberText.text = _score.toString();
		}
		
		private function setTimer()
		{			
			var timer:Timer = new Timer(_levelTimeDelay, 1);
			timer.addEventListener(TimerEvent.TIMER, onTimer);
			timer.start();
		}
		
		private function onTimer(event:TimerEvent):void
		{
			if(_mode != GAMEMODE_GAME) return;
			
			createCoin(Math.random()*256+32, 0, Math.round(Math.random()*5+1));
			
			if(_levelTimeDelay>500) _levelTimeDelay -= 10;
			setTimer();
		}
		
		public function createStaticBox(x:Number, y:Number, sizex:Number, sizey:Number):void
		{
			var bodyDef:b2BodyDef = new b2BodyDef();
			bodyDef.position.Set(x/32, y/32);
			
			var box:b2PolygonShape = new b2PolygonShape();
			box.SetAsBox(sizex/32, sizey/32);
			
			var fixtureDef:b2FixtureDef = new b2FixtureDef();
			fixtureDef.shape = box;
			
			var body:b2Body = _b2World.CreateBody(bodyDef);
			body.CreateFixture(fixtureDef);				
		}
		
		public function createStageWall():void
		{
			createStaticBox(16, 240, 16, 280);
			createStaticBox(304, 240, 16, 280);
			createStaticBox(160, 466, 160, 16);
		}
		
		public function createCoin(x:Number, y:Number, index:Number):void
		{
			var bodyDef:b2BodyDef = new b2BodyDef();
			bodyDef.position.Set(x/32, y/32);
			bodyDef.type = b2Body.b2_dynamicBody;
			bodyDef.angularDamping = 0.2;
			
			var circle:b2CircleShape = new b2CircleShape();
			var radius:Number=11 + index;			
			circle.SetRadius(radius/32);
			
			var fixtureDef:b2FixtureDef = new b2FixtureDef();
			fixtureDef.shape = circle;
			fixtureDef.density = 1;
			fixtureDef.friction = 0.1;
			
			var massData:b2MassData = new b2MassData();
			circle.ComputeMass(massData, 1);
			
			var body:b2Body = _b2World.CreateBody(bodyDef);
			body.CreateFixture(fixtureDef);
			body.SetMassData(massData);
			
			_b2CoinContainer.push(body);
			
			var coin:CCoin = new CCoin();
			coin.x=x;
			coin.y=y;
			coin._index = index;
			coin.gotoAndStop(index);	
			
			addChild(coin);
			_coinContainer.push(coin);
			
			setChildIndex(_scoreText, numChildren-1);
			setChildIndex(_scoreNumberText, numChildren-1);
		}
		
		public function createB2World():void
		{
			_b2Gravity = new b2Vec2(0, 98/32);
			_b2World = new b2World(_b2Gravity, true);
		}		  
		
		private function calcCoinCount(index:Number, selectedCoin:Array):void
		{			
			for(var i:Number=0;i<_coinContainer.length;i++)
			{
				if( i == index ) continue;
				
				// 추가 되어 있지 않을 경우에만 처리한다.
				var flag:Number = 0;
				for(var j:Number=0;j<selectedCoin.length;j++)
				{
					if(selectedCoin[j]==i)
					{
						flag = 1;
						break;
					}
				}
				if(flag==1)	continue;				
				if(_coinContainer[index]._index !== _coinContainer[i]._index) continue;
					
				var dx:Number = _coinContainer[index].x - _coinContainer[i].x;
				var dy:Number = _coinContainer[index].y - _coinContainer[i].y;
				var dist:Number = (_coinContainer[index]._index + 12) + (_coinContainer[i]._index + 12) + 5;
				if( dx * dx + dy * dy <  dist * dist)
				{
					selectedCoin.push(i);					
					calcCoinCount(i, selectedCoin);				
				}
			}
		}
		
		private function onMouseUp(event:MouseEvent):void
		{
			if(_mode == GAMEMODE_TITLE)
			{
				setGame();
				_mode = GAMEMODE_GAME;
				return;
			}
			
			if(_mode == GAMEMODE_GAMEOVER)
			{
				gotoAndPlay(1, "Title");
				_mode = GAMEMODE_TITLE;
				return;
			}
		}
		
		private function onMouseDown(event:MouseEvent):void
		{
			if(_mode != GAMEMODE_GAME) return;
			
			// 터치된 코인 판단.
			var selectedCoin:Array = new Array();
			for(var i:Number=0;i<_coinContainer.length;i++)
			{
				var dx:Number = stage.mouseX - _coinContainer[i].x;
				var dy:Number = stage.mouseY - _coinContainer[i].y;
				if( dx * dx + dy * dy < (_coinContainer[i]._index + 12) * (_coinContainer[i]._index + 12) )
				{
					selectedCoin.push(i);
					break;
				}
			}
			
			if(selectedCoin.length==0) return;
			
			// 해당 코인과 연결된 코인의 수를 조사
			calcCoinCount(selectedCoin[0], selectedCoin);
			
			var index:Number =_coinContainer[selectedCoin[0]]._index;			
			if((index==1 || index==3 || index==5) && selectedCoin.length<5)	return;
			if((index==2 || index==4 || index==6) && selectedCoin.length<2)	return;
						
			// 먼저 선택된 오브젝트의 중앙 지점에 새로운 동전을 만든다.
			// 이미 500원일 경우엔 더 이상 만들 필요가 없으므로 스킵
			if(index<6)
			{
				var cx:Number=0, cy:Number=0;
				for(i=0;i<selectedCoin.length;i++)					
				{
					cx += _coinContainer[selectedCoin[i]].x;
					cy += _coinContainer[selectedCoin[i]].y;
				}
				
				cx /= selectedCoin.length;
				cy /= selectedCoin.length;
				
				createCoin(cx, cy, index + 1);
			}
			
			// 점수 계산
			switch(index)
			{
				case 1:	_score += 1 * (1 + (selectedCoin.length - 5)*0.3); break;
				case 2:	_score += 5 * (1 + (selectedCoin.length - 2)*0.6); break;
				case 3:	_score += 10 * (1 + (selectedCoin.length - 5)*0.3); break;
				case 4:	_score += 50 * (1 + (selectedCoin.length - 2)*0.6); break;
				case 5:	_score += 100 * (1 + (selectedCoin.length - 5)*0.3); break;
				case 6:	_score += 500 * (1 + (selectedCoin.length - 2)*0.6); break;
			}
			_score = Math.round(_score);
			_scoreNumberText.text = _score.toString();
			
			// 해당 오브젝트를 모두 제거
			for(i=0;i<selectedCoin.length;i++)					
			{	
				_coinContainer[selectedCoin[i]]._destroyed = true;
			}
		}
		
		private function gameOver():void
		{
			for(var i:Number = 0; i<_coinContainer.length;i++)
			{
				_coinContainer[i]._destroyed = true;
			}
			
			//setChildIndex(_scoreText, 0);
			//setChildIndex(_scoreNumberText, 0);
			_scoreText.alpha = 0;
			_scoreNumberText.alpha = 0;
			
			gotoAndStop(1, "GameOver");
			
			_mode = GAMEMODE_GAMEOVER;
			_gameoverScoreNumberText.text = _score.toString();	
			
			// 기록 전달!
			var loader:URLLoader = new URLLoader();
			loader.dataFormat = URLLoaderDataFormat.VARIABLES;
			loader.load(new URLRequest("http://www.meteorstudio.com/rank/record.php?GameCode=CM&Score=" + _score + "&uid=" + _facebookuid));
		}
		
		private function createEffect(x:Number, y:Number)
		{
			var effect:CEffect = new CEffect;
			effect.x = x;
			effect.y = y;
			
			addChild(effect);
			
			_effectContainer.push(effect);
		}
		
		private function onEnterFrame(event:Event):void	
		{		
			if(_mode==GAMEMODE_TITLE)
			{
				if(currentFrame == 60) gotoAndPlay(1, "Title");
				return;
			}
			
			_b2World.Step( 1/30, 6, 2 );
			_b2World.ClearForces();
			
			//for(var i:Number = 0; i<_coinContainer.length;i++)
			for(var i in _coinContainer)
			{				
				if(_coinContainer[i]._destroyed == true)
				{
					// 이펙트 생성
					createEffect(_coinContainer[i].x, _coinContainer[i].y);
					
					removeChild(_coinContainer[i]);
					_b2World.DestroyBody(_b2CoinContainer[i]);	
					_coinContainer.splice(i, 1);
					_b2CoinContainer.splice(i, 1);
					i--;
					continue;
				}				
				
				_coinContainer[i].x = _b2CoinContainer[i].GetPosition().x*32;
				_coinContainer[i].y = _b2CoinContainer[i].GetPosition().y*32;
				_coinContainer[i].rotation = _b2CoinContainer[i].GetAngle() * 57.29577951;
				
				// 선택 가능한지 판단
				var selectedCoin:Array = new Array();
				selectedCoin.push(i);					
				calcCoinCount(i, selectedCoin);
				
				if((_coinContainer[i]._index==1 || _coinContainer[i]._index==3 || _coinContainer[i]._index==5) && selectedCoin.length>=5)	_coinContainer[i].setFilterOn();
				else
				if((_coinContainer[i]._index==2 || _coinContainer[i]._index==4 || _coinContainer[i]._index==6) && selectedCoin.length>=2)	_coinContainer[i].setFilterOn();
				else
				_coinContainer[i].setFilterOff();
				
				// 게임오버 검사
				if(_coinContainer[i]._creatTime > 0) 
				{
					_coinContainer[i]._creatTime -= 0.032;	
				}
				else
				{				
					if(_coinContainer[i].y < 48) gameOver();
				}
			}
			
			for(i in _effectContainer)
			{
				_effectContainer[i]._lifeTime -= 0.032;
				if(_effectContainer[i]._lifeTime < 0)
				{
					removeChild(_effectContainer[i]);
					_effectContainer.splice(i, 1);
					i--;
				}
			}
		}
	}
}